clear all
close all

%%% Add paths for matlab

addpath('')


%%% Noise. This is the observational noise, standard deviation not variance.
	noiseRun = 0.5; % The actual noise of your data (so that the correct dataset can be loaded further on). Note, this is only used to source your data, it is not passed onto other variables for computation.

	noiseStart = 0.5;%0.1; % The first discrete noise value. To hold noise fixed at a value, set noiseStart to be the same as noiseEnd.

	noiseBy = 0.1; % The increments of noise values.

	noiseEnd = 0.5;%1; % The last discrete noise value. To hold noise fixed at a value, set noiseStart to be the same as noiseEnd.
%%%

%%% ODE parameters. Lotka-Volterra has 4 parameters.
	alphaStart = 1;
	alphaBy = 0.5;
	alphaEnd = 20;

	betaStart = 1;
	betaBy = 0.5;
	betaEnd = 20;

	psiStart = 1;
	psiBy = 0.5;
	psiEnd = 20;

	deltaStart = 1;
	deltaBy = 0.5;
	deltaEnd = 20;
%%%

%%% Should annealing of the gradient mismatch parameter be used? 1 for yes, 0 for no. Default is no and thus the parameter is sampled in the Gibbs routine

	annealingUsed = 0;
%%%

%%% GP hyperparameter discrete values (assuming here rbf kernel)

	GP.discrete{1}=0.1:0.1:1; % (rbf variance hyperparameter)
	GP.discrete{2}=5:5:50;    %(rbf lengthscale hyperparameter)
%%%

%%% Set Seed. Record seeds for reproducibility. Suggest using a random draw from a large vector of integers, say 1 to 1 million and store the first N (for each dataset you want to perform the parameter inference on) somewhere so that the choice of seed is unbiased, but still recorded.

	seed = 714430; % for example
	rng(seed);
%%%


sigma2=noiseRun^2;                    % noise variance
sigma=noiseRun;
noise = num2str(noiseRun);
datasetNumber = num2str(1);

samptime = 0:0.2:2;

%%% Change load path and name for dataset. Below works with example dataset included.

	load(['LV Noise ' noise ' Dataset ' num2str(datasetNumber) '.mat'])
%%%

y = data; % Taking noisy observations from already generated dataset

% Data:
Data.y=y; % observations
Data.samptime=samptime; % the sample time



% ODE:
ODE.fun=@LV; % ODE Model
ODE.num=4;   % the number of the parameters
ODE.discrete{1}=alphaStart:alphaBy:alphaEnd;
ODE.discrete{2}=betaStart:betaBy:betaEnd;
ODE.discrete{3}=psiStart:psiBy:psiEnd;
ODE.discrete{4}=deltaStart:deltaBy:deltaEnd;


ODE.initial = [alphaStart betaStart psiStart deltaStart];             


ODE.prior{1}=@(x) unifpdf(x,0,20);% prior for ODE parameters 
ODE.prior{2}=@(x) unifpdf(x,0,20);
ODE.prior{3}=@(x) unifpdf(x,0,20);
ODE.prior{4}=@(x) unifpdf(x,0,20);

%ODE.prior{1}=@(x) gampdf(x,4,0.5);% prior for ODE parameters 
%ODE.prior{2}=@(x) gampdf(x,4,0.5);
%ODE.prior{3}=@(x) gampdf(x,4,0.5);
%ODE.prior{4}=@(x) gampdf(x,4,0.5);


% GP:
GP.fun=@GPcov; % GP covariance function: c(t,t')
GP.fun_d=@dGPcov; % GP derivative d{c(t,t')}/dt
GP.fun_dd=@ddGPcov; %GP derivative d^2{c(t,t')}/dtdt'

%%% Important: The observational noise std is placed at the end of GP hyperparameter variable. Hence, GP.num = number of hyperparameters + 1

	GP.num=3; % the number of the GP hyperparameters (put the noise std to the end of the hyperparameter vector)
%%%



GP.prior{1}=@(x) unifpdf(x,0,100);%prior for GP hyperparameters
GP.prior{2}=@(x) unifpdf(x,0,100);
GP.prior{3}=@(x) gampdf(x,1,1);

%%% The following if statement allows for the noise to be held fixed or not, depending is noiseStart = noiseEnd or not. ENSURE that GP.discrete{number of GP hyperparameters + 1} is same length as the number of discretised values for the GP hyperparameters.

	if (noiseStart==noiseEnd)

		GP.discrete{3} = [noiseStart noiseStart noiseStart noiseStart noiseStart noiseStart noiseStart noiseStart noiseStart noiseStart];

	else

		GP.discrete{3} = noiseStart:noiseBy:noiseEnd;

	end
%%%



%%% GammaAGM is the gradient mismatch parameter for the AGM method, see equation 13 in "Controversy in mechanistic modelling with Gaussian processes, ICML 2015". The proposed values begin at 32 and continue towards 0 in Log2 increments. IMPORTANT: the startSlackValue needs to be large. When the first ODE parameters are sampled in the Gibbs routine, if the slack value is originally small, then the routine cannot leave the initialised ODE parameters, which can cause numerical instabilities. Having a large slack to initialise with, means the program can sample ODE parameters that "make sense" and once it does, the program is no longer at risk of these numerical instabilities from choosing an initial ODE parameter very far away from the truth.

	incs = 20;
	Base = 2;
	startSlackValue = 32;

	store = [1];
	for i=1:incs-1
		store(i+1) = store(1)*(Base^i);
	end
		clear i



	GammaAGM.discrete{1} = startSlackValue./store;
	GammaAGM.initial = GammaAGM.discrete{1}(1);
	GammaAGM.prior{1} = @(x) unifpdf(x,0,50);
	GammaAGM.num = 1;
%%%






%%%%%%%%%% Initialization of X by using GP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[MeanX,StdX]=FitGP(y,GP,samptime);


GP.initial_X{1}=MeanX; % initial mean of X
GP.initial_X{2}=StdX; % initial std of X
GP.initial_X{3}=20; % the number of discretized bins for X
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

GP.initial=[1 10 0.5]; % initial values of GP hyperparameters NOTE that the final value here is for the observational error standard deviation. These values can be started from where the user wishes.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%
% Iter:
Iteration.total=500; % the number of the total iterations
Iteration.sub=10;   % the number of the sub iterations


[samptheta, samphyper, SAMPLEX, sampgammaAGM]= ode_ComGP_solveAGM(Data, ODE, GP, Iteration, GammaAGM, annealingUsed);

%%% Create save files for variables of interest. Recommended: Save whole workspace, save samptheta (in case of specific interest later), save sampgammaAGM (in case of specific interest later).

	save(['LV AGM Noise True Dataset ' num2str(datasetNumber) ' Noise ' noise ' Uniform ODE Prior.mat']) 

	save(['LV AGM Noise True Theta Dataset ' num2str(datasetNumber) ' Noise ' noise ' Uniform ODE Prior.mat'],'samptheta') 

	save(['LV AGM Noise True Gamma Dataset ' num2str(datasetNumber) ' Noise ' noise ' Uniform ODE Prior.mat'],'sampgammaAGM') 
%%%


